unit utype;

interface

uses
   Windows,Classes, SysUtils, Messages, Controls, Forms,Graphics, TypInfo,
   DsgnIntf,Proxy,buttons,menus;

type
  TProjectState=(psNew,psOpen,psSave,psChange);

  TProjectInfo = class(TComponent)
  private
     fProjectName,fProjectPath:string;
     fProjectState : TProjectState;
     fUnitItems:TStringList;
     fFormItems:TStringList;
     fProjectSource:TStringList;
     fMainForm:string;
     fTitle:string;
     fHelpFile:string;
     function  GetStrValue(const Str:string):string;
     procedure SetTitle(TitleName:string);
     procedure SetHelpFile(HelpName:string);
     function  GetStrPos(const Str:string):Integer;
     procedure GetMainForm;
  protected
     procedure SetUnitItems(Value:TStringList);
     procedure SetFormItems(Value:TStringList);
  public
     constructor Create(AOwner: TComponent); override;
     destructor Destroy; override;
     property HelpFile : string read fHelpFile write SetHelpFile;
     property ProjectName : string read fProjectName write fProjectName;
     property MainForm : string read fMainForm write fMainForm;
     property ProjectPath : string read fProjectPath write fProjectPath;
     property ProjectState : TProjectState read fProjectState write fProjectState;
     property UnitItems : TStringList read fUnitItems write SetUnitItems;
     property FormItems : TStringList read fFormItems write SetFormItems;
     property ProjectSource : TStringList read fProjectSource write fProjectSource;
     property Title : string read fTitle write SetTitle;
     procedure UpdateProject(Sender:TObject);
  end;

{ Expand the SetDesiging method of a component. This method
  is protected, but casting the component to TExposeComponent
  allows other objects to put the component into design mode.
  This is important for the component form. }
type
  TExposeComponent = class(TComponent)
  public
    procedure SetDesigning(Value: Boolean);
  end;

  TProxyDesigner = class(TFormDesigner)
  private
    fMethods: TStrings;   { Refers to the method list in ComponentForm. }
    fGrabHandles: TGrabHandles;
    fPoint:TPoint;
    fGrabList: TList;  //  ۳Ʈ ǥϱ   Grab 
    fSelectList: TList;  //  ۳Ʈ  ϱ  List
    fMultiSelected: Boolean;
    function  GetFocusedControl(Msg: TMsg): TControl;
    procedure FindNextControl;
    procedure ProcessMouseDown(Sender:TControl;Message:TMessage);
    procedure ProcessRMouseDown(Sender:TControl;Message:TMessage);
    procedure ProcessMouseMove(Sender:TControl;Message:TMessage);
    procedure ProcessDblClick(Sender:TControl;Message:TMessage);
    procedure ProcessMouseUp(Sender:TControl;Message:TMessage);
    procedure ProcessKeyDown(Message:TMessage);
    procedure ProcessKeyUp(Message:TMessage);
    procedure AddComponentList(Component:TComponent);
    procedure DeleteControl(Control: TControl;Next:Boolean);
  public
    constructor Create(Owner:TForm);
    destructor Destroy; override;
    function  IsDesignMsg(Sender: TControl; var Message: TMessage): Boolean; override;
    procedure Modified; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure PaintGrid; override;
    procedure ValidateRename(AComponent: TComponent;const CurName, NewName: string); override;
    function  CreateMethod(const Name: string; TypeData: PTypeData): TMethod; override;
    function  GetMethodName(const Method: TMethod): string; override;
    procedure GetMethods(TypeData: PTypeData; Proc: TGetStrProc); override;
    function  GetPrivateDirectory: string; override;
    function  MethodExists(const Name: string): Boolean; override;
    procedure RenameMethod(const CurName, NewName: string); override;
    procedure ShowMethod(const Name: string); override;
    procedure SelectComponent(Instance: TPersistent); override;
    function  GetComponentName(Component: TComponent):string; override;
    function  CreateComponent(ComponentClass: TComponentClass;Parent:TComponent;
              Left,Top,Width,Height:Integer):TComponent;override;
    { Easy access to the list of methods. }
    property  Methods: TStrings read fMethods;
    function  MethodFromAncestor(const Method: TMethod): Boolean; override;
    function  UniqueName(const BaseName:string):string;override;
    procedure GetSelections(List: TComponentList); override;
    procedure SetSelections(List: TComponentList); override;
    procedure GetComponentNames(TypeData: PTypeData; Proc: TGetStrProc); override;
    function  GetComponent(const Name: string): TComponent; override;
    function  GetObject(const Name: string): TPersistent; override;
    function  GetObjectName(Instance: TPersistent): string; override;
    procedure GetObjectNames(TypeData: PTypeData; Proc: TGetStrProc); override;
   // function IsComponentLinkable(Component: TComponent): Boolean; override;
   // procedure MakeComponentLinkable(Component: TComponent); override;
    function  GetRoot: TComponent; override;
    procedure Revert(Instance: TPersistent; PropInfo: PPropInfo); override;
    function  GetIsDormant: Boolean; override;
    function  HasInterface: Boolean; override;
    function  HasInterfaceMember(const Name: string): Boolean; override;
    procedure AddInterfaceMember(const MemberText: string); override;

    procedure GetComponentList;
    procedure DeleteSelectList;
    procedure UpdateGrabHandle;
  end;

implementation

uses
  WinTypes,Editor,ObjectInspec,MainForm,Dialogs,uconst;

constructor TProjectInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fUnitItems := TStringList.Create;
  fFormItems := TStringList.Create;
  fProjectSource := TStringList.Create;
  ProjectSource.OnChange:=UpdateProject;
end;

destructor TProjectInfo.Destroy;
begin
  fUnitItems.Free;
  fFormItems.Free;
  fProjectSource.Free;
  inherited Destroy;
end;

procedure TProjectInfo.SetUnitItems(Value: TStringList);
begin
   fUnitItems.Assign(Value);
end;

procedure TProjectInfo.SetFormItems(Value: TStringList);
begin
   fUnitItems.Assign(Value);
end;

procedure TProjectInfo.GetMainForm;
var
  I,Pos1,Pos2:Integer;
begin
   I:=GetStrPos('Application.CreateForm');
   if I<>-1 then begin
      Pos1:=Pos(',',ProjectSource[I]);
      Pos2:=Pos(')',ProjectSource[I]);
      if Pos1>0 then fMainForm:=Trim(Copy(ProjectSource[I],Pos1+1,Pos2-Pos1-1));
   end;
end;

procedure TProjectInfo.UpdateProject(Sender:TObject);
var
   I:Integer;
begin
   for I:=0 to ProjectSource.Count-1 do begin
       if Pos('Application.Title',ProjectSource[I])>0 then  fTitle:=GetStrValue(ProjectSource[I]);
       if Pos('Application.HelpFile',ProjectSource[I])>0 then  fHelpFile:=GetStrValue(ProjectSource[I]);
   end;
   GetMainForm;
end;

// Ʈ ҽ ڿ Ҵ (̸) ش.
function TProjectInfo.GetStrValue(const Str:string):string;
var
  Pos1,Pos2:Integer;
  S:String;
begin
   Pos1:=Pos(':=',Str);
   Pos2:=Pos(';',Str);
   S:=Trim(Copy(Str,Pos1+2,Pos2-Pos1-2));
   Result:=Copy(S,2,Length(S)-2);
end;

procedure TProjectInfo.SetTitle(TitleName:string);
var
   I:Integer;
begin
   I:=GetStrPos('Application.Title');
   if I<>-1 then begin
        ProjectSource[I]:='  Application.Title :='+''''+TitleName+''''+';';
        ProjectSource.SaveToFile(ProjectName);
        Exit;
   end;
   I:=GetStrPos('Application.Initialize;');
   if I=-1 then Exit;
   ProjectSource.Insert(I+1,'  Application.Title :='+''''+TitleName+''''+';');
   ProjectSource.SaveToFile(ProjectName);
end;

procedure TProjectInfo.SetHelpFile(HelpName:string);
var
   I:Integer;
begin
   I:=GetStrPos('Application.HelpFile');
   if I<>-1 then begin
        ProjectSource[I]:='  Application.HelpFile :='+''''+HelpName+''''+';';
        ProjectSource.SaveToFile(ProjectName);
        Exit;
   end;
   I:=GetStrPos('Application.Initialize;');
   if I=-1 then Exit;
   ProjectSource.Insert(I+1,'  Application.HelpFile :='+''''+HelpName+''''+';');
   ProjectSource.SaveToFile(ProjectName);
end;

// Ʈ ҽ Ư ڿ ִ  ˷ش.
function TProjectInfo.GetStrPos(const Str:string):Integer;
var
  I:Integer;
begin
   Result:=-1;
   for I:=0 to ProjectSource.Count-1 do
       if Pos(Str,ProjectSource[I])>0 then begin
          Result:=I;
          Exit;
       end;
end;

procedure TExposeComponent.SetDesigning(Value: Boolean);
begin
  inherited SetDesigning(Value);
end;

constructor TProxyDesigner.Create(Owner:TForm);
begin
   Form:= Owner;
   fGrabList:= TList.Create;
   fSelectList:= TList.Create;
end;

destructor TProxyDesigner.Destroy;
begin
   fGrabList.Free;
   fSelectList.Free;
   inherited;
end;

procedure TProxyDesigner.Modified;
begin
end;

procedure TProxyDesigner.GetSelections(List: TComponentList);
begin
end;

procedure TProxyDesigner.SetSelections(List: TComponentList);
begin
end;

procedure TProxyDesigner.GetComponentNames(TypeData: PTypeData; Proc: TGetStrProc);
begin
end;

function TProxyDesigner.GetComponent(const Name: string): TComponent;
begin
end;

function TProxyDesigner.GetObject(const Name: string): TPersistent;
begin
end;

function TProxyDesigner.GetObjectName(Instance: TPersistent): string;
begin
end;

procedure TProxyDesigner.GetObjectNames(TypeData: PTypeData; Proc: TGetStrProc);
begin
end;

function TProxyDesigner.GetRoot: TComponent;
begin
end;

procedure TProxyDesigner.Revert(Instance: TPersistent; PropInfo: PPropInfo);
begin
end;

function TProxyDesigner.GetIsDormant: Boolean;
begin
end;

function TProxyDesigner.HasInterface: Boolean;
begin
end;

function TProxyDesigner.HasInterfaceMember(const Name: string): Boolean;
begin
end;

procedure TProxyDesigner.AddInterfaceMember(const MemberText: string);
begin
end;

function TProxyDesigner.GetComponentName(Component:TComponent):string;
begin
   Result:=Component.Name;
end;

procedure TProxyDesigner.Notification(AComponent: TComponent;
                                      Operation: TOperation);
begin
{   case Operation of
      opInsert:
      begin
        if (AComponent is TGrabHandle) then Exit;
        AddComponentList(AComponent);
        FMainForm.SelectedComponent:=AComponent;
      end;
      opRemove:
      begin
        if (AComponent is TGrabHandle) then Exit;
        GetComponentList;
      end;
   end;}
end;

procedure TProxyDesigner.PaintGrid;
var
 i, j, HStart, HEnd, VStart, VEnd, HCount, VCount, X, Y, SIZE: integer;
begin
  SIZE := 8;
  with Form do begin
      Canvas.Pen.color := clBlack;
      Canvas.Pen.width := 1;

      VStart := (Canvas.ClipRect.Top div SIZE) * SIZE;
      VEnd := (Canvas.ClipRect.Bottom div SIZE) * SIZE;
      HStart := (Canvas.ClipRect.Left div SIZE) * SIZE;
      HEnd := (Canvas.ClipRect.Right div SIZE) * SIZE;
      HCount := (HEnd - HStart) div SIZE + 1;
      VCount := (VEnd - VStart) div SIZE + 1;

      Y := VStart;
      for i := 1 to VCount do
      begin
         X := HStart;
         for j := 1 to HCount do
         begin
             Canvas.moveto(X, Y);
             Canvas.LineTo(X+1, Y+1);
             X := X + SIZE;
         end;
         Y := Y + SIZE;
      end;
    end;
end;

procedure TProxyDesigner.ValidateRename(AComponent: TComponent;
                               const CurName, NewName: string);
begin
end;

{ Create a new method called Name, whose type is TypeData. Assign a unique
  identifier for the method by using its index in the string list. Offset the
  index by 1, so that index 0 can be interpreted as a nil pointer. Thus, the
  first method in the list has index 1. After creating the method, show it
  to the user. }
function TProxyDesigner.CreateMethod(const Name: string; TypeData: PTypeData): TMethod;
begin
  if not MethodExists(Name) then
    Methods.AddObject(Name, TObject(TypeData));
  Result.Data := nil;
  Result.Code := Pointer(Methods.IndexOf(Name) + 1);
  ShowMethod(Name);
end;

{ Call Proc for each method whose type matches TypeData. }
procedure TProxyDesigner.GetMethods(TypeData: PTypeData; Proc: TGetStrProc);
var
  I: Integer;
begin
  with Methods do
    for I := 0 to Count-1 do
      if Objects[I] = TObject(TypeData) then
        Proc(Strings[I]);
end;

{ Get the name of a method whose pointer is TMethod. Look up the
  method name by its index in the method's Code field. Remember that
  indexes are offset by one. }
function TProxyDesigner.GetMethodName(const Method: TMethod): string;
begin
  if Method.Code = nil then
    Result := ''
  else
    Result := Methods.Strings[Integer(Method.Code) - 1];
end;

{ Return whether a method exists with the given Name. }
function TProxyDesigner.MethodExists(const Name: string): Boolean;
begin
  Result := Methods.IndexOf(Name) >= 0;
end;

{ Rename a method from CurName to NewName. }
procedure TProxyDesigner.RenameMethod(const CurName, NewName: string);
var
  Index: Integer;
begin
  Index := Methods.IndexOf(CurName);
  Methods.Strings[Index] := NewName;
end;

{ Show a method to the user by highlighting it in the list box. }
procedure TProxyDesigner.ShowMethod(const Name: string);
begin
  //ComponentForm.MethodList.ItemIndex := Methods.IndexOf(Name);
end;

function TProxyDesigner.GetPrivateDirectory: string;
begin
end;

{$ifndef VER80}
{ No form inheritance in the debugging designer. }
function TProxyDesigner.MethodFromAncestor(const Method: TMethod): Boolean;
begin
  Result := False;
end;
{$endif}

// ۳Ʈ  ,   ۳Ʈ Ѵ.
procedure TProxyDesigner.FindNextControl;
var
  Name:string;
  Index:Integer;
begin
   if TProxyForm(Form).ComponentList.Count <= 0 then begin
      ObjectInspector.ComponentList.ItemIndex:=0;
      ObjectInspector.ComponentListClick(self);
      Exit;
   end;
   Name:= TProxyForm(Form).ComponentList[TProxyForm(Form).ComponentList.Count - 1];
   Index:= ObjectInspector.ComponentList.Items.IndexOf(Name);
   if Index <> -1 then
      ObjectInspector.ComponentList.ItemIndex:= Index
   else ObjectInspector.ComponentList.ItemIndex:=0;
   ObjectInspector.ComponentListClick(self);
end;

// Control ϰ,  Ʈ Ѵ.
procedure TProxyDesigner.DeleteControl(Control: TControl;Next:Boolean);
var
   Name:string;
   Index:Integer;
begin
   if (Control = nil) or (Control = Form) then Exit;
   Name:= Control.Name + ':' + Control.ClassName;
   Index:= TProxyForm(Form).ComponentList.IndexOf(Name);
   if Index <> -1 then  TProxyForm(Form).ComponentList.Delete(Index);
   ObjectInspector.ComponentList.Items.Delete(ObjectInspector.ComponentList.Items.IndexOf(Name));
   EditorForm.WriteComponentCode(Control.Name,
       TComponentClass(Control.ClassType),wrDelete);
   Control.Free;
   if Next then FindNextControl;  //  ۳Ʈ ã Ѵ.
end;

//   ۳Ʈ Ѵ.
procedure TProxyDesigner.DeleteSelectList;
var
  I:Integer;
begin
   TProxyForm(Form).SelectControl:= nil;
   for I:= 0 to fSelectList.Count-1 do
      DeleteControl(fSelectList[I], False);
   FindNextControl;
end;

procedure TProxyDesigner.SelectComponent(Instance: TPersistent);
var
  I:Integer;
begin
  { Create the grab handles, so the user can move & resize. }
  if not (fMultiSelected) and (fGrabHandles<>nil) then begin
     for I:=fGrabList.Count-1 downto 0 do begin
        TGrabHandles(fGrabList[I]).Free;
        fGrabList.Delete(I);
        fSelectList.Delete(I);
     end;
  end;
  try
    fGrabHandles :=TGrabHandles.Create(TControl(Instance));
    fGrabList.Add(TPersistent(fGrabHandles));
    fSelectList.Add(Instance);
  except
  on  EAccessViolation do
      Exit;
  end;
  FMainForm.SelectedComponent:=TComponent(Instance);
end;

// Ͻ  ۳Ʈ Ѵ.
// ComponentClass:  ۳Ʈ Ŭ
// Parent:  ۳Ʈ θ ۳Ʈ
// Left,Top,Width,Height:  ۳Ʈ ġ
function TProxyDesigner.CreateComponent(ComponentClass: TComponentClass;Parent:TComponent;
             Left,Top,Width,Height:Integer):TComponent;
var
  Component: TComponent;
  Control :TControl;
begin
   Result:=nil;
   if ComponentClass=nil then Exit;
   Component:=ComponentClass.Create(Form);
   if Component is TControl then  // ð ۳Ʈ 
   begin
      Control := TControl(Component);
      Control.Name:=UniqueName(ComponentClass.ClassName);
      if Left>0 then  Control.Left:=Left;
      if Top>0 then   Control.Top:=Top;
      if Width>0 then Control.Width:=Width;
      if Height>0 then  Control.Height:=Height;
      Control.Parent := TWinControl(Parent);
   end
   else begin // Create a wrapper control for nonvisual components.
      Control := TWrapperControl.Create(Form, Component);
      // Wrapper ̸ Wrapper+۳Ʈ ̸ Ѵ.
      Component.Name:=UniqueName(ComponentClass.ClassName);
      Control.Name:='Wrapper'+Component.Name;
      if Left>0 then  Control.Left:=Left;
      if Top>0 then   Control.Top:=Top;
      if Width>0 then Control.Width:=Width;
      if Height>0 then  Control.Height:=Height;
      Control.Parent := TWinControl(Parent);
   end;
   //  ۳Ʈ ޼  ʵ csDesigning flag True Ѵ.
   TExposeComponent(Control).SetDesigning(True);
   //  ۳Ʈ  ڵ ۼ
   EditorForm.WriteComponentCode(Component.Name,ComponentClass,wrInsert);
   TProxyForm(Form).SelectControl:= Control;
   TProxyForm(Form).GrabHandles := fGrabHandles;
   AddComponentList(Component);
   // Ʈ νƮ ش ۳Ʈ Ӽ  ش.
   if Control.ClassType=TWrapperControl then
      ObjectInspector.DisplayProperty(Control,(Control as TWrapperControl).Component)
   else ObjectInspector.DisplayProperty(nil,Control);
   // ۳Ʈ ȷƮ  ȭǥ ư Ѵ.
   if not FMainForm.ShiftKeyed then
      TSpeedButton(FMainForm.PageControl1.ActivePage.Controls[0]).Down:= True;
   Result:=Component;
end;

{ǥ  Ģ Ͽ ۳Ʈ   ̸ Ѵ.  <Ÿ><>
  ϴµ, <Ÿ> ۳Ʈ Ŭ ̸ 'T'  ̰, <>
 ̸ ϰ   ̴.}
function TProxyDesigner.UniqueName(const BaseName:string):string;
var
  I:Integer;
  Fmt:string;
begin
   if BaseName[1] in ['t','T'] then
      Fmt:=Copy(BaseName,2,255)+'%d'
   else Fmt:=BaseName+'%d';
   { ̸   ,  ڸ  Ѵ.}
   for I:=1 to High(Integer) do begin
      Result:=Format(Fmt,[I]);
     if Form.FindComponent(Result)=nil then Exit;
   end;
end;

//  ִ ۳Ʈ  Ѵ.
procedure TProxyDesigner.GetComponentList;
var
   I:integer;
begin
    ObjectInspector.ComponentList.Clear;
    ObjectInspector.ComponentList.Items.Add(Form.Name+':TForm');
    with Form do  begin
    for I:=0 to  ComponentCount-1 do
        if (Components[I].ClassType<>TGrabHandle) and (Components[I].ClassType<>TWrapperControl) then
           ObjectInspector.ComponentList.Items.Add(Components[I].Name+':'+Components[I].ClassName)
    end;
end;

// Ͻ   Ʈ  Ѵ.
function TProxyDesigner.GetFocusedControl(Msg: TMsg): TControl;
var
   FocusControl: TControl;
   I:Integer;
   P:TPoint;
begin
   FocusControl:=nil;
   if Msg.HWnd=Form.Handle then begin
      //  Ǵ WinControl ƴ ٸ ۳Ʈ  
      FocusControl:=Form.ControlAtPos(Form.ScreenToClient(Msg.Pt),True);
      if FocusControl=nil then  FocusControl:=Form;
   end
   else
   begin
       for I:=0 to Form.ComponentCount-1 do
          if (Form.Components[I] is TWinControl) and
             (Msg.HWnd=TWinControl(Form.Components[I]).Handle) then
             FocusControl:=TControl(Form.Components[I]);
   end;
   Result:=FocusControl;
end;

// ο ۳Ʈ Object Inspector ۳Ʈ Ͽ ߰Ѵ.
procedure TProxyDesigner.AddComponentList(Component:TComponent);
begin
   ObjectInspector.ComponentList.Items.Add(Component.Name+':'+Component.ClassName);
   TProxyForm(Form).ComponentList.Add(Component.Name+':'+Component.ClassName);
end;

// Ͻ  ߻ϴ  ޼ óѴ.
function TProxyDesigner.IsDesignMsg(Sender: TControl;var Message: TMessage): Boolean;
begin
  Result := True;
  case Message.Msg of
     WM_KEYDOWN: ProcessKeyDown(Message);
     WM_KEYUP: ProcessKeyUp(Message);
     WM_LBUTTONDOWN: ProcessMouseDown(Sender,Message);
     WM_RBUTTONDOWN: ProcessRMouseDown(Sender,Message);
     WM_MOUSEMOVE: ProcessMouseMove(Sender,Message);
     WM_LBUTTONDBLCLK: ProcessDblClick(Sender,Message);
     WM_LBUTTONUP: ProcessMouseUp(Sender,Message);
     else Result:= False;
  end;
end;

// Ͻ  ߻ϴ MouseDown ޼ óѴ.
procedure TProxyDesigner.ProcessRMouseDown(Sender:TControl;Message:TMessage);
var
  Menu:TPopupMenu;
  Po:TPoint;
// ˾ ޴ ޴ ׸ ߰Ѵ.
procedure AddMenuItem(Menu:TPopupMenu;ItemName:string);
var
   NewMenuItem:TMenuItem;
begin
   NewMenuItem:=TMenuItem.Create(Application);
   NewMenuItem.Caption:=ItemName;
   NewMenuItem.OnClick:= TProxyForm(Form).OnMenuClick;
   Menu.Items.Add(NewMenuItem);
end;
begin
   Menu:= TPopupMenu.Create(Form);
   AddMenuItem(Menu, 'Bring to Front');
   AddMenuItem(Menu, 'Send to Back');
   AddMenuItem(Menu, 'Delete');
   GetCursorPos(Po);
   Menu.Popup(Po.X, Po.Y);
  // Form.Dispatch(Message);
end;

// Ͻ  ߻ϴ MouseDown ޼ óѴ.
procedure TProxyDesigner.ProcessMouseDown(Sender:TControl;Message:TMessage);
var
  Control:TComponent;
begin
   // Ŀ ġ ִ ۳Ʈ ϰ,  Ӽ Inspector ش.
   Control:= Sender;
   if (Control<>nil) and (TProxyForm(Form).SelectControl <> Sender) then begin
      if Control.ClassType=TWrapperControl then
         ObjectInspector.DisplayProperty(Control,(Control as TWrapperControl).Component)
      else if  Control.ClassType<>TGrabHandle then
         ObjectInspector.DisplayProperty(nil,Control);
   end;
   TProxyForm(Form).SelectControl:= Sender;
   Form.Dispatch(Message);   //  ޼ Ѵ.
end;

// Ͻ  ߻ϴ MouseMove ޼ óѴ.
procedure TProxyDesigner.ProcessMouseMove(Sender:TControl;Message:TMessage);
begin
   Form.Dispatch(Message);
end;

// Ͻ  ߻ϴ MouseUp ޼ óѴ.
procedure TProxyDesigner.ProcessMouseUp(Sender:TControl;Message:TMessage);
begin
   Form.Dispatch(Message);
end;

// Ͻ  ߻ϴ DblClick ޼ óѴ.
procedure TProxyDesigner.ProcessDblClick(Sender:TControl;Message:TMessage);
begin
   with ObjectInspector do begin
      SetFocus;
      PageControl1.ActivePage:= ObjectInspector.TabEvent;
      EventBox.ItemIndex:=0;
      EventBoxClick(self);
      edEventDblClick(self);
   end;
end;

// Ͻ  ߻ϴ KeyDown ޼ óѴ.
procedure TProxyDesigner.ProcessKeyDown(Message:TMessage);
begin
   case TWMKey(Message).CharCode of
       VK_DELETE:
          DeleteSelectList;
       VK_F11:
          ObjectInspector.SetFocus;
       VK_F12:
          FMainForm.ToggleFormAndUnit;
      // VK_SHIFT :
      //    fMultiSelected:= True
   end;
  // Form.Dispatch(Message);
end;

// Ͻ  ߻ϴ KeyUp ޼ óѴ.
procedure TProxyDesigner.ProcessKeyUp(Message:TMessage);
begin
   case TWMKey(Message).CharCode of
       VK_SHIFT :
          fMultiSelected:= False;
   end;
  // Form.Dispatch(Message);
end;

procedure TProxyDesigner.UpdateGrabHandle;
begin
   fGrabHandles.Update;
end;

end.
